define("ember-makeup/services/makeup", ["exports", "ember-makeup/config", "ember-makeup/utils/stylesheet/reader", "p-event"], function (_exports, _config, _reader, _pEvent) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.default = void 0;

  var _dec, _class, _descriptor, _temp;

  function _initializerDefineProperty(target, property, descriptor, context) { if (!descriptor) return; Object.defineProperty(target, property, { enumerable: descriptor.enumerable, configurable: descriptor.configurable, writable: descriptor.writable, value: descriptor.initializer ? descriptor.initializer.call(context) : void 0 }); }

  function _applyDecoratedDescriptor(target, property, decorators, descriptor, context) { var desc = {}; Object.keys(descriptor).forEach(function (key) { desc[key] = descriptor[key]; }); desc.enumerable = !!desc.enumerable; desc.configurable = !!desc.configurable; if ('value' in desc || desc.initializer) { desc.writable = true; } desc = decorators.slice().reverse().reduce(function (desc, decorator) { return decorator(target, property, desc) || desc; }, desc); if (context && desc.initializer !== void 0) { desc.value = desc.initializer ? desc.initializer.call(context) : void 0; desc.initializer = undefined; } if (desc.initializer === void 0) { Object.defineProperty(target, property, desc); desc = null; } return desc; }

  function _initializerWarningHelper(descriptor, context) { throw new Error('Decorating class property failed. Please ensure that ' + 'proposal-class-properties is enabled and set to use loose mode. ' + 'To use proposal-class-properties in spec mode with decorators, wait for ' + 'the next major version of decorators in stage 2.'); }

  // @TODO: This fails, when the primary host does not depend on `ember-makeup`
  // directly. What should the correct behavior be? I think we should throw a
  // build error, similar to `ember-cli-resolve-asset`.
  function removeNode(node) {
    if (node.parentNode) {
      node.parentNode.removeChild(node);
    }
  }

  let MakeupService = (_dec = Ember.computed('linkElement'), (_class = (_temp = class MakeupService extends Ember.Service.extend(Ember.Evented) {
    constructor(...args) {
      super(...args);
      this.customPropertyPrefix = _config.default.options.customPropertyPrefix;
      this.contextClassNamePrefix = _config.default.options.contextClassNamePrefix;
      this.themePaths = _config.default.themePaths;
      this.supportsCustomProperties = Boolean(window.CSS && CSS.supports('--foo: var(--bar)'));

      _initializerDefineProperty(this, "linkElement", _descriptor, this);

      this.isReady = false;
    }

    get stylesheetReader() {
      return (0, _reader.makeStylesheetReader)({
        customPropertyPrefix: this.customPropertyPrefix,
        contextClassNamePrefix: this.contextClassNamePrefix
      });
    }

    get properties() {
      if (!this.linkElement || !this.linkElement.sheet) return undefined;
      const sheet = this.linkElement.sheet;
      return this.stylesheetReader(sheet);
    }

    willDestroy() {
      super.willDestroy();
      this.destroyLinkElement();
    }

    async createLinkElement(href) {
      const linkElement = document.createElement('link');
      linkElement.setAttribute('rel', 'stylesheet');
      linkElement.setAttribute('href', href);
      document.head.appendChild(linkElement);

      try {
        await (0, _pEvent.default)(linkElement, 'load'); // Only destroy the old link element _after_ the new one loaded
        // successfully, to avoid FOUC.

        this.destroyLinkElement();
        this.linkElement = linkElement;
      } catch (error) {
        // When loading the new link element failed, still destroy the old link
        // element.
        this.destroyLinkElement(); // But also cleanup the new link element.

        removeNode(linkElement);
        throw error;
      }
    }

    destroyLinkElement() {
      if (this.linkElement) {
        removeNode(this.linkElement);
        this.linkElement = undefined;
      }
    }

    resolveContext(key) {
      if (!this.isReady) return undefined;
      const context = this.getPropertyValue(key);
      (true && !(Boolean(context)) && Ember.assert("Could not resolve context '".concat(key, "'."), Boolean(context))); // eslint-disable-next-line @typescript-eslint/no-non-null-assertion

      return this.prefixContext(context);
    }

    prefixContext(context) {
      return "".concat(this.contextClassNamePrefix).concat(context);
    }

    getPropertyValue(key, context) {
      if (!this.properties) return undefined;

      if (context) {
        return this.properties.contextual[context] && this.properties.contextual[context][key];
      }

      return this.properties.contextless[key];
    }

    async setTheme(themeName) {
      (true && !(themeName in this.themePaths) && Ember.assert("'".concat(themeName, "' is not in the list of known themes: ").concat(Object.keys(this.themePaths).join(', ')), themeName in this.themePaths));
      this.isReady = false;
      await this.createLinkElement(this.themePaths[themeName]);
      this.isReady = true;
      this.trigger('theme-change');
    }

  }, _temp), (_descriptor = _applyDecoratedDescriptor(_class.prototype, "linkElement", [Ember._tracked], {
    configurable: true,
    enumerable: true,
    writable: true,
    initializer: null
  }), _applyDecoratedDescriptor(_class.prototype, "properties", [_dec], Object.getOwnPropertyDescriptor(_class.prototype, "properties"), _class.prototype)), _class));
  _exports.default = MakeupService;
});