define("ember-makeup/utils/stylesheet/reader", ["exports", "ember-makeup/utils/css"], function (_exports, _css) {
  "use strict";

  Object.defineProperty(_exports, "__esModule", {
    value: true
  });
  _exports.makeStylesheetReader = void 0;

  const makeStylesheetReader = ({
    customPropertyPrefix,
    contextClassNamePrefix
  }) => sheet => {
    const getContextFromSelector = selector => (0, _css.unescape)(selector).slice(contextClassNamePrefix.length + 1);

    const getKeyFromCustomProperty = property => (0, _css.unescape)(property).slice(customPropertyPrefix.length + 2);

    const contextless = {};
    const contextual = {};

    for (const rule of sheet.cssRules) {
      let properties = contextless;

      if (rule.selectorText !== ':root') {
        const context = getContextFromSelector(rule.selectorText);

        if (!contextual[context]) {
          contextual[context] = {};
        }

        properties = contextual[context];
      }

      for (const property of rule.style) {
        if (!property.startsWith('--')) continue;
        properties[getKeyFromCustomProperty(property)] = rule.style.getPropertyValue(property).trim();
      }
    }

    return {
      contextless,
      contextual
    };
  };

  _exports.makeStylesheetReader = makeStylesheetReader;
});