"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const path_1 = require("path");
const broccoli_multifilter_1 = __importDefault(require("broccoli-multifilter"));
const fast_glob_1 = __importDefault(require("fast-glob"));
const make_dir_1 = __importDefault(require("make-dir"));
const async_fs_1 = require("../../../lib/utils/async-fs");
const compatibility_css_1 = require("../../postcss/compatibility-css");
var InputPath;
(function (InputPath) {
    InputPath[InputPath["Themes"] = 0] = "Themes";
    InputPath[InputPath["Styles"] = 1] = "Styles";
})(InputPath || (InputPath = {}));
class BroccoliConfigCreatorCSSCompatibility extends broccoli_multifilter_1.default {
    constructor(themes, styles, options) {
        super([themes, styles], options);
        this.themes = [];
        this.styles = [];
        this.styleExtension = 'makeup.json';
    }
    scanFiles() {
        return __awaiter(this, void 0, void 0, function* () {
            [this.themes, this.styles] = yield Promise.all([
                fast_glob_1.default('**/*.json', { cwd: this.inputPaths[InputPath.Themes] }),
                fast_glob_1.default(`**/*.${this.styleExtension}`, {
                    cwd: this.inputPaths[InputPath.Styles]
                })
            ]);
        });
    }
    prefixPath(inputPath, filePath) {
        return path_1.join(this.inputPaths[inputPath], filePath);
    }
    generateCompatibilityCSS(theme, usages) {
        return __awaiter(this, void 0, void 0, function* () {
            const css = compatibility_css_1.generateCompatibilityCSS(theme, usages);
            return css.toString();
        });
    }
    buildTheme(themePath, outputDirectory) {
        return __awaiter(this, void 0, void 0, function* () {
            const absoluteThemePath = this.prefixPath(InputPath.Themes, themePath);
            const theme = JSON.parse(yield async_fs_1.readFile(absoluteThemePath, 'utf8'));
            const absoluteStylePaths = this.styles.map(stylePath => this.prefixPath(InputPath.Styles, stylePath));
            yield Promise.all(this.styles.map((path, i) => __awaiter(this, void 0, void 0, function* () {
                const absolutePath = absoluteStylePaths[i];
                const outputPath = path_1.join(outputDirectory, `${path.slice(0, -(this.styleExtension.length + 1))}.${theme.name}.css`);
                const { usages } = JSON.parse(yield async_fs_1.readFile(absolutePath, 'utf8'));
                const compatibilityCSS = yield this.generateCompatibilityCSS(theme, usages);
                yield make_dir_1.default(path_1.dirname(outputPath));
                yield async_fs_1.writeFile(outputPath, compatibilityCSS, 'utf8');
            })));
            return { dependencies: [absoluteThemePath, ...absoluteStylePaths] };
        });
    }
    build() {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.scanFiles();
            yield this.buildAndCache(this.themes, this.buildTheme);
        });
    }
}
exports.BroccoliConfigCreatorCSSCompatibility = BroccoliConfigCreatorCSSCompatibility;
